! This file is part of HiggsBounds
!  -KW
!******************************************************************
module channels	
!******************************************************************
 implicit none
 integer,allocatable :: t1_order(:)
 
 contains
 !**********************************************************	
 subroutine setup_channels
 !**********************************************************
 ! puts together a list of processes to be checked by HiggsBounds 
 ! sets number of processes to be checked (= ntot)
 !**********************************************************
  use usefulbits, only : nH,debug,whichexpt, & !input
		       & ntot,pr  
  use S95tables, only : nLEPtable1,nLEPtable2,nTEVtable1,nTEVtable2, &!input
		       & S95_t2 !input
  implicit none
  !-----------------------------------internal	
  integer :: ntable1,ntable2,nt 
  logical :: proper_fill_pr
  !-------------------------------------------

  select case(whichexpt)
  case('onlyL')
   ntable1=nLEPtable1
   ntable2=nLEPtable2 
  case('onlyT')
   ntable1=nTEVtable1
   ntable2=nTEVtable2 
  case('LandT')
   ntable1=nLEPtable1+nTEVtable1
   ntable2=nLEPtable2+nTEVtable2
  case('singH')
   ntable1=nLEPtable1+nTEVtable1 
   ntable2=0   
  case default 
   write(*,*)'error in setup_channels' ; call flush(6) ; stop
  end select			
  
  ! number of processes to test (nb won't all be valid)			
  	
  proper_fill_pr=.True.
  if(proper_fill_pr)then	  
   ntot = nH*ntable1 + nH**2*ntable2  
   call fill_pr
  else
   !allows you to select which processes you want included individually 
   !this is only designed for testing the Tevatron tables
   nt=8
   ntot=nH*nt    
   call fill_pr_select(nt)
  endif
  
  if(debug)then
    write(*,*)'ntot=',ntot ; call flush(6)  
    call print_init_stuff_for_debugging
  endif

  contains
 
  !******************************************
  subroutine print_init_stuff_for_debugging
  !******************************************
  logical :: templogical
  integer :: n
  ! This section is just to assist with debugging
    write(21,*)whichexpt,nH

    do n=1,ntot

      if(pr(n)%ttype.eq.1)then
       templogical=.False.
      elseif(.not.S95_t2(pr(n)%tlist)%needs_Mh2_gt_2Mh1)then
       templogical=.True.
      else
       templogical=.False.
      endif
      
      write(21,'(5I6,1L4)')n,pr(n)%tlist,pr(n)%ttype, &
                                 &  pr(n)%findj,pr(n)%findi, &
                                 &  templogical  
    enddo  
  end subroutine print_init_stuff_for_debugging

 end subroutine setup_channels
		
 !******************************************************************		
 subroutine check_channels(t,r)
 !******************************************************************
 !Checks each process against predicted experimental bounds
 !For some of the processes with high stat. sensitivity, then 
 !compares to the measured experimental bounds
 !******************************************************************
  use usefulbits, only: nH,ntot,debug,pr, & !input
                      & dataset,results, & 
                      & whichexpt		      
  use S95tables
  implicit none
  !--------------------------------------input	
  type(dataset) :: t	
  !-------------------------------------output
  type(results) :: r
  !-----------------------------------internal	
  double precision, allocatable :: predratio(:),fact(:)		
  integer, allocatable :: ncomb(:)
  integer :: ii,n,k,m,x
  integer :: rep
  integer :: mlratio 
  double precision, allocatable :: Mhj_av(:)
  !-------------------------------------------		
	
  allocate(predratio(ntot),fact(ntot),ncomb(ntot))
  allocate(Mhj_av(ntot))  

  rep=size(r%obsratio)
  !if((size(r%chan).ne.rep).or.(size(r%allowed95).ne.rep))then
  ! write(*,*)'wrong input to subroutine check_channels in module channels'
  ! stop
  !endif			
  
  if( t%gooddataset )then ! labels a physical parameter point	   	
   do n=1,ntot  
    call calcfact(pr(n),t,fact(n),Mhj_av(n),ncomb(n))! calculates the theoretical factor that will be compared to the expt result  
            !For LEP tables, fact is S_theo (see HB paper for full definition)      
    call check_against_bound(pr(n),t,fact(n),Mhj_av(n),predratio(n),2) !checks each process against
                                                         !predicted experimental bounds   
   enddo			   
       	
   !if(debug) call print_stuff_for_debugging

   ! i=1 uses channel with highest stat. sensitivity
   ! compare with fact(mlratio) with experimental S95 value	  
   do ii=1,rep	 
    mlratio=maxloc(predratio,dim=1)
    
    if(predratio(mlratio).gt.0.0D0)then          	    
		  
      ! setting the channel with the highest stat. sens. to -1 so that
      ! the next highest is selected next time round	   
      predratio(mlratio)= -1.0D0	

      call check_against_bound(pr(mlratio),t,fact(mlratio),Mhj_av(mlratio),r%obsratio(ii),1)  	

      r%chan(ii)=mlratio
      r%ncombined(ii)=ncomb(mlratio)      
    else ! none of the processes apply: label with zero
      r%chan(ii)=0	   
      r%obsratio(ii)=0  
      r%ncombined(ii)=0
    endif

    if((r%chan(ii).gt.0).and.(r%obsratio(ii).ge.1.0D0))then
      r%allowed95(ii)=0 !excluded
    elseif(r%obsratio(ii).lt.0.0D0)then
      r%obsratio =  0.0D0
      r%ncombined=  0   
      r%chan=  0
      r%allowed95= -1
    else
      r%allowed95(ii)=1
    endif	

    !if(r%obsratio(ii).lt.0.0D0)write(*,*)'hello A',r%obsratio(ii),r%chan(ii)

   enddo  
  else ! gooddataset=False
   r%obsratio =  0.0D0
   r%ncombined=  0   
   r%chan=  0
   r%allowed95= -1
  endif      

  deallocate(Mhj_av)   
  deallocate(predratio) 
  deallocate(fact)      
  deallocate(ncomb)    
 
  contains
 
  !******************************************
  subroutine print_stuff_for_debugging
  !******************************************
  ! This section is just to assist with debugging

    x=2
    if(x.gt.size(t%Mh))stop 'error in debug part of subroutine check_channels' 
    write(20,*)'********************' 
    write(20,'(" * Mh:",1E16.7)')t%Mh(1)
    write(20,*)'********************' 
     !write(20,*)'x',x 
     ! BE CAREFUL ABOUT PRINTING THESE OUT: check option of whichexpt 
     ! write(20,*)'********************'  
     ! write(20,*)'t%Mh(x):',              t%Mh(x)   
     ! write(20,*)'********************'
     ! write(20,*)'t%BR_hjbb(x):',         t%BR_hjbb(x)     
     ! write(20,*)'t%BR_hjWW(x):',         t%BR_hjWW(x) 
     ! write(20,*)'t%BR_hjtautau(x):',     t%BR_hjtautau(x) 
     ! write(20,*)'t%BR_hjgaga(x):',       t%BR_hjgaga(x) 
     ! write(20,*)'********************'      
     ! write(20,*)'t%lep%XS_hjZ_ratio(x):',t%lep%XS_hjZ_ratio(x)  
     ! write(20,*)'t%tev%XS_hjb_ratio(x)', t%tev%XS_hjb_ratio(x)  
     ! write(20,*)'t%tev%XS_hj_ratio(x):', t%tev%XS_hj_ratio(x)    
     ! write(20,*)'t%tev%XS_hjZ_ratio(x):',t%tev%XS_hjZ_ratio(x)  
     ! write(20,*)'t%tev%XS_hjW_ratio(x):',t%tev%XS_hjW_ratio(x)   
     ! write(20,*)'t%tev%XS_vbf_ratio(x):',t%tev%XS_vbf_ratio(x) 
     ! write(20,*)'********************' 
     ! write(20,*)'t%tev%XS_H_SM(x)',      t%tev%XS_HZ_SM(x) 
     ! write(20,*)'t%tev%XS_Hb_SM(x)',     t%tev%XS_Hb_SM(x)
     ! write(20,*)'********************'    
     ! write(20,*)'BR_HWW_SM(x):',         t%BR_HWW_SM(x)      
     ! write(20,*)'********************'       

    write(20,*)'*    n     predratio(n)  '     
    do n=1,ntot
      write(20,'(1I6,1E16.7)')n,predratio(n)    
    enddo      
    
    !write(*,*)'********************'    
    !write(*,'(1I6,2E12.3)')maxloc(predratio,dim=1),maxval(predratio,dim=1),r%obsratio(1)
    !write(*,*)'********************'
    
!    stop'just debugging at the moment'

  end subroutine print_stuff_for_debugging
  !******************************************
 end subroutine check_channels  	
 
 !******************************************************************
 subroutine fill_pr
 !******************************************************************
 !fills the components of pr (which will keep track of the
 !characteristics of each process)
 !tries to fill the arrays in a similar order to OB, to make comparing
 !the codes easier
 !******************************************************************
  use usefulbits, only : nH,ntot,whichexpt, & !input
                       & pr !allocates, fills
  use S95tables, only : S95_t1,S95_t2 !input
  implicit none
  !-----------------------------------internal
  integer :: n,x,y,i,j
  !-------------------------------------------
    
  allocate(pr(0:ntot))	
  allocate(t1_order(lbound(S95_t1,dim=1):ubound(S95_t1,dim=1)))
 
  ! fill t1_order
  do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   t1_order(x)=x
  enddo
  call fill_t1_order_OB
  
  !use this for debugging:
  !call check_tev_tables	

  n=0

  pr(n)%tlist=0
  pr(n)%ttype=0
  pr(n)%findi=0
  pr(n)%findj=0
  
  select case(whichexpt)
  case('onlyL','LandT','singH')
   do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
    y=t1_order(x)
    do i=1,nH
      if(S95_t1(y)%expt.eq.'LEP')then
       n=n+1      
      
       pr(n)%tlist=y   !tlist is the table number i.e. which component of
                    !S95_t1 or S95_t1 to use
       pr(n)%ttype=1   !ttype is the table type 
       pr(n)%findi=i   !findi gives the i in h_i for this process
       pr(n)%findj=i   !findj gives the j in h_j for this process
      endif 
    enddo  	  
   enddo	 
  end select 

  select case(whichexpt)
  case('onlyL','LandT')
   do x=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
    do j=1,nH
      do i=1,nH
       n=n+1
       pr(n)%tlist=x
       pr(n)%ttype=2
       pr(n)%findi=i
       pr(n)%findj=j 
      enddo 
    enddo  	  
   enddo	 
  end select			
  
  select case(whichexpt)
  case('onlyT','LandT','singH')
   do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
    y=t1_order(x)
    do i=1,nH
      if(((S95_t1(y)%expt.eq.'CDF').or.(S95_t1(y)%expt.eq.' D0')) &
        .or.(S95_t1(y)%expt.eq.'TSM'))then
       n=n+1
       pr(n)%tlist=y
       pr(n)%ttype=1   
       pr(n)%findi=i   
       pr(n)%findj=i     
      endif    
    enddo  	  
   enddo	 
  end select 	

  deallocate(t1_order)
	
 end subroutine fill_pr
 
 !******************************************************************
 subroutine fill_pr_select(nt)
 !******************************************************************
 !fills the components of pr (which will keep track of the
 !characteristics of each process) according to a particular
 !list of topologies, specified by their S95_t1(y)%id
 !Only used for the purposes of testing Tevatron tables at the moment
 !******************************************************************
  use usefulbits, only : nH,ntot,whichexpt, & !input
                       & pr !allocated, filled
  use S95tables, only : S95_t1  !input
  implicit none
  !-----------------------------------internal
  integer :: n,x,i,j,nt
  !-------------------------------------------
  
  write(*,*)'WARNING: MUST TAKE OUT!!!'
  write(*,*)  
  
  if(whichexpt.ne.'onlyT')stop 'error in fill_pr_select (2)'
   
  allocate(pr(0:ntot))	
	
  n=0

  pr(n)%tlist=0
  pr(n)%ttype=0
  pr(n)%findi=0
  pr(n)%findj=0  
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(8957)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(8742)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo  
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(7081)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(5482)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo    
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(5586)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(8958)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo  
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(5624)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo
  
  do i=1,nH
   n=n+1  
   pr(n)%tlist=findxfromid(5489)
   pr(n)%ttype=1
   pr(n)%findi=i
   pr(n)%findj=i
  enddo 
  
  if(n.ne.nt) stop'error in fill_pr_select'
   
  !do n=1,ntot
  ! write(*,*)n,pr(n)%tlist,pr(n)%ttype,pr(n)%findi,pr(n)%findj
  !enddo
  
 contains
 
 !***********************************************
 function findxfromid(id)
 ! find which element x of S95_t1(x)
 ! has S95_t1(x)%id.eq.id
 !***********************************************
  implicit none
  integer :: id !input
  integer :: findxfromid !function
  integer :: y,z !internal 
  
  z=0
  do y=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if(S95_t1(y)%id.eq.id)then
    findxfromid=y
    z=z+1
   endif 
  enddo
  
  if(z.ne.1)stop'bad input to function findxfromid'
  
 end function findxfromid
 !***********************************************
 		
 end subroutine fill_pr_select

 !******************************************************************
 subroutine fill_t1_order_OB
 !******************************************************************
 ! The array t1_order is used in subroutine fill_pr to determine the 
 ! order the processes should be checked in. This subroutine (fill_t1_order_OB)
 ! fills t1_order using the value of S95_t1(x)%OBid for each topology 
 ! such that the ordering will be as consistent as possible with
 ! that used in OB's version of the code.
 ! (makes comparing the codes much easier)
 !******************************************************************
  use S95tables, only : S95_t1,S95_t2,nTEVtable1  !input
  implicit none
  !-----------------------------------internal
  integer :: n,x,y,nbeg,nend,m,ybeg,a
  integer,allocatable :: OBid_t1(:)
  !-------------------------------------------
    	
  allocate(OBid_t1(lbound(S95_t1,dim=1):ubound(S95_t1,dim=1)))

  ! first part of t1_order should be filled with LEP tables:
  y=1
  do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   OBid_t1(x)=S95_t1(x)%OBid
   if(S95_t1(x)%expt.eq.'LEP')then   
    t1_order(y)=x
    y=y+1
   endif   
  enddo
  
  ybeg=y
   
  !count the number of TEV tables with a valid OBid (i.e. > 0)
  a=0
  do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)  
   if(S95_t1(x)%expt.ne.'LEP')then 
    if(S95_t1(x)%OBid.gt.0)a=a+1
   endif
  enddo

  !fills part of array, using TEV tables with a valid OBid
  if(a.ne.0)then ! i.e.only if there's a TEV table with a valid OBid
   do y= ybeg+a-1, ybeg,-1
    m=ybeg 
    do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)  
     if(S95_t1(x)%expt.ne.'LEP')then 
      if(OBid_t1(x).gt.OBid_t1(m))then
       m=x
      endif
     endif    
    enddo
    t1_order(y)=m
    OBid_t1(m) = -2  
   enddo
  endif 
 
  !shifts TEV tables without a valid OBid to the end
  if(a.lt.nTEVtable1)then
   do y= ybeg+a, ybeg+(nTEVtable1-1)
    m=ybeg 
    do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)  
     if(S95_t1(x)%expt.ne.'LEP')then 
      if(OBid_t1(x).gt.OBid_t1(m))then
       m=x
      endif
     endif    
    enddo
    t1_order(y)=m
    OBid_t1(m) = -2  
   enddo
  endif

  !uncomment to visually check t1_order
  !do n=1,ubound(S95_t1,dim=1)
  ! write(*,*)'hello:',t1_order(n),S95_t1(t1_order(n))%OBid
  !enddo

  ! to check that t1_order is filled correctly
  ! every number n=1,ubound(S95_t1,dim=1) is used once and once only
  do n=1,ubound(S95_t1,dim=1)
   a=0
   do m=1,ubound(S95_t1,dim=1)
    if(t1_order(m).eq.n)a=a+1
   enddo
   if(a.ne.1)stop'error in subroutine fill_pr_usingOBid'
  enddo

  deallocate(OBid_t1)
	
 end subroutine fill_t1_order_OB

 !************************************************************		
 subroutine check_tev_tables
 !************************************************************
 ! prints out all the Tevatron data to a file 'debugging_tables.txt'
 ! in a format which can be easily compared to the data in OB's code's tables
 ! (see compare_tables.bat)
  use S95tables, only : S95_t1
  implicit none
  !-----------------------------------internal
  integer :: x,y,i
  double precision :: Mhi
  !-------------------------------------------

  open(10,file='debugging_tables.txt')

  do x=lbound(t1_order,dim=1),ubound(t1_order,dim=1)
     y=t1_order(x)
     if((S95_t1(y)%expt.ne.'LEP').and.(S95_t1(y)%OBid.gt.0))then
       write(10,'(" OBid of table:",1I4)')S95_t1(y)%OBid
       do i=lbound(S95_t1(y)%dat,dim=1),ubound(S95_t1(y)%dat,dim=1)
        Mhi=S95_t1(y)%Mhmin+dble(i-1)*S95_t1(y)%sep
        write(10,'(3E16.7)')Mhi,S95_t1(y)%dat(i,1),S95_t1(y)%dat(i,2)
       enddo 
     endif
  enddo 

  close(10)
  
  stop'subroutine check_tev_tables should only be used when debugging'

 end subroutine check_tev_tables
 !************************************************************

end module channels			
!******************************************************************
